#ifndef BBA_H
#define BBA_H

#include "shared.h"
#include "dfa_seq.h"

namespace bba {
    template<typename elem_t, int DIMS, typename key_t, typename data_t, typename key_cmp_t, typename key_agg_t>
    struct bba_input_t {
        range_t<record_t<elem_t, DIMS, key_t, data_t>*> records;
        size_t blockSize;
		int fanout;
		int paddedFanout;
        key_cmp_t keyCmp;
        key_agg_t keyAgg;

		bba_input_t(range_t<record_t<elem_t, DIMS, key_t, data_t>*> records, size_t blockSize, int fanout, int paddedFanout, key_cmp_t keyCmp, key_agg_t keyAgg)
			: records(records)
			, blockSize(blockSize)
			, fanout(fanout)
			, paddedFanout(paddedFanout)
			, keyCmp(keyCmp)
			, keyAgg(keyAgg) {}
    };

    template<typename elem_t, int DIMS, typename key_t, typename data_t, typename key_agg_t, int MAX_HEIGHT>
    struct bba_internal_input_t {
        range_t<record_t<elem_t, DIMS, key_t, data_t>*> records;
        size_t blockSize;
        size_t numBlocks;
        key_agg_t keyAgg;
        rtree_t<elem_t, DIMS, key_t, data_t>* rtrees;
		rtree_layout_t<MAX_HEIGHT> layout;

		bba_internal_input_t(
			range_t<record_t<elem_t, DIMS, key_t, data_t>*> records,
			size_t blockSize,
			size_t numBlocks,
			key_agg_t keyAgg,
			rtree_t<elem_t, DIMS, key_t, data_t>* rtrees,
			rtree_layout_t<MAX_HEIGHT> layout
		)
			: records(records)
			, blockSize(blockSize)
			, numBlocks(numBlocks)
			, keyAgg(keyAgg)
			, rtrees(rtrees)
			, layout(layout) {}

		bba_internal_input_t(bba_internal_input_t<elem_t, DIMS, key_t, data_t, key_agg_t, MAX_HEIGHT>&) = delete;
		
		bba_internal_input_t(bba_internal_input_t<elem_t, DIMS, key_t, data_t, key_agg_t, MAX_HEIGHT>&& other) noexcept
			: records(std::exchange(other.records, { nullptr, other.records.last }))
			, blockSize(std::move(other.blockSize))
			, numBlocks(std::move(other.numBlocks))
			, keyAgg(std::move(other.keyAgg))
			, rtrees(std::exchange(other.rtrees, nullptr))
			, layout(std::move(other.layout)) {}

		~bba_internal_input_t() {
			delete[] this->records.first;
			delete[] this->rtrees;
		}
    };
}

#endif //BBA_H
