#ifndef BP_SEQ_H
#define BP_SEQ_H

#include "bp.h"

namespace bp::seq {
	template<typename iter_t, typename cmp_t>
	void blockPartition(bp::block_partition_work_item_t<iter_t> workItem, iter_t basis, size_t blockSize, cmp_t cmp) {
		bp::block_partition_work_item_t<iter_t> nextWorkItems[2];
		int nextWorkItemCount = bp::processAndDivideWorkItem(workItem, nextWorkItems, basis, blockSize, cmp);
		for (int i = 0; i < nextWorkItemCount; ++i) {
			blockPartition(nextWorkItems[i], basis, blockSize, cmp);
		}
	}

	// A block partitioning operation under a comparator is a partial sort that divides the input into a series of blocks
	// All entries in a block in the resulting series of blocks are smaller than all entries in all following blocks
	template<typename iter_t, typename cmp_t>
	void blockPartition(iter_t first, iter_t last, size_t blockSize, cmp_t cmp) {
		bp::block_partition_work_item_t<iter_t> initialWorkItem(first, last - first);
		blockPartition(initialWorkItem, first, blockSize, cmp);
	}

	// Performs a segmented block partitioning operation
	// The input is divided into contiguous segments where a block partitioning operation is performed on each segment
	template<typename iter_t, typename cmp_t>
	void segmentedBlockPartition(
		iter_t first,
		iter_t last,
		size_t segmentSize,
		size_t blockSize,
		cmp_t cmp
	) {
		assert(segmentSize % blockSize == 0 /* segmentSize must be a multiple of blockSize */);
		iter_t i = first;
		for (; i + segmentSize < last; i += segmentSize) {
			blockPartition(i, i + segmentSize, blockSize, cmp);
		}
		if (i < last) {
			blockPartition(i, last, blockSize, cmp);
		}
	}
}

#endif //BP_SEQ_H
