#ifndef SHARED_H
#define SHARED_H

#include <cstdlib>

template<typename item_t>
struct tuple_t {
	item_t left;
	item_t right;

    __device__ __host__ bool operator==(const tuple_t<item_t>& other) {
        return this->left == other.left && this->right == other.right;
    }
};

template<typename score_t, typename data_t>
struct score_data_pair_t {
    score_t score;
    data_t data;
};

template<typename iter_t>
struct range_t {
    iter_t first;
    iter_t last;
};

template<typename elem_t, int DIMS>
struct bounding_box_t {
	elem_t min[DIMS];
	elem_t max[DIMS];
};

template<typename elem_t, int DIMS, typename key_t, typename data_t>
struct alignas(32) record_t {
	bounding_box_t<elem_t, DIMS> mbb;
	key_t key;
    union {
		data_t data;
        int childBlock;
    };
};

struct segment_t {
    size_t offset;
    size_t size;
    size_t paddedSize;
};

template<int MAX_HEIGHT>
struct rtree_layout_t {
    segment_t segments[MAX_HEIGHT];
    size_t height;
    size_t size;
    int fanout;
};

template<typename elem_t, int DIMS, typename key_t, typename data_t>
struct rtree_t {
    record_t<elem_t, DIMS, key_t, data_t> root;
    record_t<elem_t, DIMS, key_t, data_t>* records;
};

int ceilDiv(int numerator, int denominator) {
	std::div_t division = std::div(numerator, denominator);
	return division.quot + (division.rem != 0);
}

long ceilDiv(long numerator, long denominator) {
	std::ldiv_t division = std::div(numerator, denominator);
	return division.quot + (division.rem != 0);
}

long long ceilDiv(long long numerator, long long denominator) {
	std::lldiv_t division = std::div(numerator, denominator);
	return division.quot + (division.rem != 0);
}

#endif //SHARED_H
